use std::collections::HashMap;

use crate::{enums::ProtocolVersion, state::AtemState};

pub trait DeserializedCommand {
    fn apply_to_state(&self, state: &mut AtemState) -> Vec<String>;
}

pub trait DeserializableCommand: DeserializedCommand {
    fn get_raw_name(&self) -> &'static str;
    fn get_minimum_version(&self) -> ProtocolVersion;
}

pub trait SerializableCommand {
    fn payload(&self, version: ProtocolVersion) -> Vec<u8>;
}

pub trait BasicWritableCommand: SerializableCommand {
    fn get_raw_name(&self) -> &'static str;
    fn get_minimum_version(&self) -> ProtocolVersion;
}

pub trait WritableCommand: BasicWritableCommand {
    fn get_mask_flag(&self) -> HashMap<String, f64>;
    fn get_flag(&self) -> f64;
    fn set_flag(&mut self, flag: f64);
}

pub trait SymmetricalCommand: DeserializableCommand + SerializableCommand {}
