pub enum Model {
    Unknown = 0x00,
    TVS = 0x01,
    OneME = 0x02,
    TwoME = 0x03,
    PS4K = 0x04,
    OneME4K = 0x05,
    TwoME4K = 0x06,
    TwoMEBS4K = 0x07,
    TVSHD = 0x08,
    TVSProHD = 0x09,
    TVSPro4K = 0x0a,
    Constellation = 0x0b,
    Constellation8K = 0x0c,
    Mini = 0x0d,
    MiniPro = 0x0e,
    MiniProISO = 0x0f,
    MiniExtreme = 0x10,
    MiniExtremeISO = 0x11,
}

impl Default for Model {
    fn default() -> Self {
        Model::Unknown
    }
}

pub enum ProtocolVersion {
    Unknown = 0,
    V7_2 = 0x00020016,   // 2.22 // TODO - verify this is correct
    V7_5_2 = 0x0002001b, // 2.27 // The naming of this may be off
    V8_0 = 0x0002001c,   // 2.28
    V8_0_1 = 0x0002001d, // 2.29
    V8_1_1 = 0x0002001e, // 2.30
}

impl Default for ProtocolVersion {
    fn default() -> Self {
        ProtocolVersion::Unknown
    }
}

pub enum TransitionStyle {
    MIX = 0x00,
    DIP = 0x01,
    WIPE = 0x02,
    DVE = 0x03,
    STING = 0x04,
}

pub enum TransitionSelection {
    Background = 1 << 0,
    Key1 = 1 << 1,
    Key2 = 1 << 2,
    Key3 = 1 << 3,
    Key4 = 1 << 4,
}

pub enum DVEEffect {
    SwooshTopLeft = 0,
    SwooshTop = 1,
    SwooshTopRight = 2,
    SwooshLeft = 3,
    SwooshRight = 4,
    SwooshBottomLeft = 5,
    SwooshBottom = 6,
    SwooshBottomRight = 7,

    SpinCCWTopRight = 13,
    SpinCWTopLeft = 8,
    SpinCCWBottomRight = 15,
    SpinCWBottomLeft = 10,
    SpinCWTopRight = 9,
    SpinCCWTopLeft = 12,
    SpinCWBottomRight = 11,
    SpinCCWBottomLeft = 14,

    SqueezeTopLeft = 16,
    SqueezeTop = 17,
    SqueezeTopRight = 18,
    SqueezeLeft = 19,
    SqueezeRight = 20,
    SqueezeBottomLeft = 21,
    SqueezeBottom = 22,
    SqueezeBottomRight = 23,

    PushTopLeft = 24,
    PushTop = 25,
    PushTopRight = 26,
    PushLeft = 27,
    PushRight = 28,
    PushBottomLeft = 29,
    PushBottom = 30,
    PushBottomRight = 31,

    GraphicCWSpin = 32,
    GraphicCCWSpin = 33,
    GraphicLogoWipe = 34,
}

pub enum MacroAction {
    Run = 0,
    Stop = 1,
    StopRecord = 2,
    InsertUserWait = 3,
    Continue = 4,
    Delete = 5,
}

pub enum ExternalPortType {
    Unknown = 0,
    SDI = 1,
    HDMI = 2,
    Component = 4,
    Composite = 8,
    SVideo = 16,
    XLR = 32,
    AESEBU = 64,
    RCA = 128,
    Internal = 256,
    TSJack = 512,
    MADI = 1024,
    TRSJack = 2048,
}

pub enum InternalPortType {
    External = 0,
    Black = 1,
    ColorBars = 2,
    ColorGenerator = 3,
    MediaPlayerFill = 4,
    MediaPlayerKey = 5,
    SuperSource = 6,
    // Since V8_1_1
    ExternalDirect = 7,

    MEOutput = 128,
    Auxiliary = 129,
    Mask = 130,
    // Since V8_1_1
    MultiViewer = 131,
}

const SOURCE_AVAILABILITY_NONE: isize = 0;
const SOURCE_AVAILABILITY_AUXILIARY: isize = 1 << 0;
const SOURCE_AVAILABILITY_MULTIVIEWER: isize = 1 << 1;
const SOURCE_AVAILABILITY_SUPERSOURCE_ART: isize = 1 << 2;
const SOURCE_AVAILABILITY_SUPERSOURCE_BOX: isize = 1 << 3;
const SOURCE_AVAILABILITY_KEY_SOURCE: isize = 1 << 4;
const SOURCE_AVAILABILITY_AUXILIARY_1: isize = 1 << 5;
const SOURCE_AVAILABILITY_AUXILIARY_2: isize = 1 << 6;
pub enum SourceAvailability {
    None = SOURCE_AVAILABILITY_NONE,
    Auxiliary = SOURCE_AVAILABILITY_AUXILIARY,
    Multiviewer = SOURCE_AVAILABILITY_MULTIVIEWER,
    SuperSourceArt = SOURCE_AVAILABILITY_SUPERSOURCE_ART,
    SuperSourceBox = SOURCE_AVAILABILITY_SUPERSOURCE_BOX,
    KeySource = SOURCE_AVAILABILITY_KEY_SOURCE,
    Auxiliary1 = SOURCE_AVAILABILITY_AUXILIARY_1,
    Auxiliary2 = SOURCE_AVAILABILITY_AUXILIARY_2,
    All = SOURCE_AVAILABILITY_AUXILIARY
        | SOURCE_AVAILABILITY_MULTIVIEWER
        | SOURCE_AVAILABILITY_SUPERSOURCE_ART
        | SOURCE_AVAILABILITY_SUPERSOURCE_BOX
        | SOURCE_AVAILABILITY_KEY_SOURCE
        | SOURCE_AVAILABILITY_AUXILIARY_1
        | SOURCE_AVAILABILITY_AUXILIARY_2, // Auxiliary | Multiviewer | SuperSourceArt | SuperSourceBox | KeySource | Auxiliary1 | Auxiliary2
}

const ME_AVAILABILITY_NONE: isize = 0;
const ME_AVAILABILITY_ME_1: isize = 1 << 0;
const ME_AVAILABILITY_ME_2: isize = 1 << 1;
const ME_AVAILABILITY_ME_3: isize = 1 << 2;
const ME_AVAILABILITY_ME_4: isize = 1 << 3;
pub enum MeAvailability {
    None = ME_AVAILABILITY_NONE,
    Me1 = ME_AVAILABILITY_ME_1,
    Me2 = ME_AVAILABILITY_ME_2,
    Me3 = ME_AVAILABILITY_ME_3,
    Me4 = ME_AVAILABILITY_ME_4,
    All = ME_AVAILABILITY_ME_1 | ME_AVAILABILITY_ME_2 | ME_AVAILABILITY_ME_3 | ME_AVAILABILITY_ME_4,
}

pub enum BorderBevel {
    None = 0,
    InOut = 1,
    In = 2,
    Out = 3,
}

pub enum IsAtKeyFrame {
    None = 0,
    A = 1 << 0,
    B = 1 << 1,
    RunToInfinite = 1 << 2,
}

pub enum Pattern {
    LeftToRightBar = 0,
    TopToBottomBar = 1,
    HorizontalBarnDoor = 2,
    VerticalBarnDoor = 3,
    CornersInFourBox = 4,
    RectangleIris = 5,
    DiamondIris = 6,
    CircleIris = 7,
    TopLeftBox = 8,
    TopRightBox = 9,
    BottomRightBox = 10,
    BottomLeftBox = 11,
    TopCentreBox = 12,
    RightCentreBox = 13,
    BottomCentreBox = 14,
    LeftCentreBox = 15,
    TopLeftDiagonal = 16,
    TopRightDiagonal = 17,
}

#[derive(Clone, Copy)]
pub enum MixEffectKeyType {
    Luma = 0,
    Chroma = 1,
    Pattern = 2,
    DVE = 3,
}

pub enum FlyKeyKeyFrame {
    None = 0,
    A = 1,
    B = 2,
    Full = 3,
    RunToInfinite = 4,
}

pub enum FlyKeyDirection {
    CentreOfKey = 0,
    TopLeft = 1,
    TopCentre = 2,
    TopRight = 3,
    MiddleLeft = 4,
    MiddleCentre = 5,
    MiddleRight = 6,
    BottomLeft = 7,
    BottomCentre = 8,
    BottomRight = 9,
}

pub enum SuperSourceArtOption {
    Background,
    Foreground,
}

pub enum TransferMode {
    NoOp,
    Write,
    Clear,
    WriteAudio = 256,
}

pub enum VideoMode {
    N525i5994NTSC = 0,
    P625i50PAL = 1,
    N525i5994169 = 2,
    P625i50169 = 3,

    P720p50 = 4,
    N720p5994 = 5,
    P1080i50 = 6,
    N1080i5994 = 7,
    N1080p2398 = 8,
    N1080p24 = 9,
    P1080p25 = 10,
    N1080p2997 = 11,
    P1080p50 = 12,
    N1080p5994 = 13,

    N4KHDp2398 = 14,
    N4KHDp24 = 15,
    P4KHDp25 = 16,
    N4KHDp2997 = 17,

    P4KHDp5000 = 18,
    N4KHDp5994 = 19,

    N8KHDp2398 = 20,
    N8KHDp24 = 21,
    P8KHDp25 = 22,
    N8KHDp2997 = 23,
    P8KHDp50 = 24,
    N8KHDp5994 = 25,

    N1080p30 = 26,
    N1080p60 = 27,
}

impl Default for VideoMode {
    fn default() -> Self {
        VideoMode::N525i5994NTSC
    }
}

pub enum TransferState {
    Queued,
    Locked,
    Transferring,
    Finished,
}

pub enum MediaSourceType {
    Still = 1,
    Clip,
}

pub enum AudioMixOption {
    Off = 0,
    On = 1,
    AudioFollowVideo = 2,
}

pub enum AudioSourceType {
    ExternalVideo,
    MediaPlayer,
    ExternalAudio,
}

pub enum StreamingError {
    None,
    InvalidState = 1 << 4,
    Unknown = 1 << 15,
}

pub enum StreamingStatus {
    Idle = 1 << 0,
    Connecting = 1 << 1,
    Streaming = 1 << 2,
    Stopping = 1 << 5, // + Streaming
}

pub enum RecordingError {
    None = 1 << 1,
    NoMedia = 0,
    MediaFull = 1 << 2,
    MediaError = 1 << 3,
    MediaUnformatted = 1 << 4,
    DroppingFrames = 1 << 5,
    Unknown = 1 << 15,
}

pub enum RecordingStatus {
    Idle = 0,
    Recording = 1 << 0,
    Stopping = 1 << 7,
}

pub enum RecordingDiskStatus {
    Idle = 1 << 0,
    Unformatted = 1 << 1,
    Active = 1 << 2,
    Recording = 1 << 3,

    Removed = 1 << 5,
}

pub enum FairlightAudioMixOption {
    Off = 1,
    On = 2,
    AudioFollowVideo = 4,
}

pub enum FairlightInputConfiguration {
    Mono = 1,
    Stereo = 2,
    DualMono = 4,
}

pub enum FairlightAnalogInputLevel {
    Microphone = 1,
    ConsumerLine = 2,
    // [Since(ProtocolVersion.V8_1_1)]
    ProLine = 4,
}

pub enum FairlightAudioSourceType {
    Mono = 0,
    Stereo = 1,
}

pub enum FairlightInputType {
    EmbeddedWithVideo = 0,
    MediaPlayer = 1,
    AudioIn = 2,
    MADI = 4,
}

const MULTI_VIEWER_LAYOUT_DEFAULT: isize = 0;
const MULTI_VIEWER_LAYOUT_TOP_LEFT_SMALL: isize = 1;
const MULTI_VIEWER_LAYOUT_TOP_RIGHT_SMALL: isize = 2;
const MULTI_VIEWER_LAYOUT_BOTTOM_LEFT_SMALL: isize = 4;
const MULTI_VIEWER_LAYOUT_BOTTOM_RIGHT_SMALL: isize = 8;
pub enum MultiViewerLayout {
    Default = MULTI_VIEWER_LAYOUT_DEFAULT,
    TopLeftSmall = MULTI_VIEWER_LAYOUT_TOP_LEFT_SMALL,
    TopRightSmall = MULTI_VIEWER_LAYOUT_TOP_RIGHT_SMALL,
    ProgramBottom = MULTI_VIEWER_LAYOUT_TOP_LEFT_SMALL | MULTI_VIEWER_LAYOUT_TOP_RIGHT_SMALL, // TopLeftSmall | TopRightSmall
    BottomLeftSmall = MULTI_VIEWER_LAYOUT_BOTTOM_LEFT_SMALL,
    ProgramRight = MULTI_VIEWER_LAYOUT_TOP_LEFT_SMALL | MULTI_VIEWER_LAYOUT_BOTTOM_LEFT_SMALL, // TopLeftSmall | BottomLeftSmall
    BottomRightSmall = MULTI_VIEWER_LAYOUT_BOTTOM_RIGHT_SMALL,
    ProgramLeft = MULTI_VIEWER_LAYOUT_TOP_RIGHT_SMALL | MULTI_VIEWER_LAYOUT_BOTTOM_RIGHT_SMALL, // TopRightSmall | BottomRightSmall
    ProgramTop = MULTI_VIEWER_LAYOUT_BOTTOM_LEFT_SMALL | MULTI_VIEWER_LAYOUT_BOTTOM_RIGHT_SMALL, // BottomLeftSmall | BottomRightSmall
}
