use crate::enums;

pub trait UpstreamKeyerBase: UpstreamKeyerTypeSettings {
    fn get_upstream_keyer_id(&self) -> f64;
    fn get_can_fly_key(&self) -> bool;
    fn get_fill_source(&self) -> f64;
    fn set_fill_source(&mut self, source: f64);
    fn get_cut_source(&self) -> f64;
    fn set_cut_source(&mut self, source: f64);
}

pub trait UpstreamKeyerTypeSettings {
    fn get_mix_effect_key_type(&self) -> enums::MixEffectKeyType;
    fn set_mix_effect_key_type(&mut self, key_type: enums::MixEffectKeyType);
    fn get_fly_enabled(&self) -> bool;
    fn set_fly_enabled(&mut self, enabled: bool);
}

pub trait UpstreamKeyerMaskSettings {
    fn get_mask_enabled(&self) -> bool;
    fn set_mask_enabled(&mut self, enabled: bool);
    fn get_mask_top(&self) -> f64;
    fn set_mask_top(&mut self, mask: f64);
    fn get_mask_bottom(&self) -> f64;
    fn set_mask_bottom(&mut self, mask: f64);
    fn get_mask_left(&self) -> f64;
    fn set_mask_right(&mut self, mask: f64);
}

pub trait UpstreamKeyerDVEBase: UpstreamKeyerMaskSettings {
    fn get_size_x(&self) -> f64;
    fn set_size_x(&mut self, size_x: f64);
    fn get_size_y(&self) -> f64;
    fn set_size_y(&mut self, size_y: f64);
    fn get_position_x(&self) -> f64;
    fn set_position_x(&mut self, position_x: f64);
    fn get_position_y(&self) -> f64;
    fn set_position_y(&mut self, position_y: f64);
    fn get_rotation(&self) -> f64;
    fn set_rotation(&mut self, rotation: f64);

    fn get_border_outer_width(&self) -> f64;
    fn set_border_outer_width(&mut self, width: f64);
    fn get_border_inner_width(&self) -> f64;
    fn set_border_inner_width(&mut self, width: f64);
    fn get_border_outer_softness(&self) -> f64;
    fn set_border_outer_softness(&mut self, softness: f64);
    fn get_border_inner_softness(&self) -> f64;
    fn set_border_inner_softness(&mut self, softness: f64);
    fn get_border_bevel_softness(&self) -> f64;
    fn set_border_bevel_softness(&mut self, softness: f64);
    fn get_border_bevel_position(&self) -> f64;
    fn set_border_bevel_position(&mut self, position: f64);

    fn get_border_opacity(&self) -> f64;
    fn set_border_opacity(&mut self, opacity: f64);
    fn get_border_hue(&self) -> f64;
    fn set_border_hue(&mut self, hue: f64);
    fn get_border_saturation(&self) -> f64;
    fn set_border_saturation(&mut self, saturation: f64);
    fn get_border_luma(&self) -> f64;
    fn set_border_luma(&mut self, luma: f64);

    fn get_light_source_direction(&self) -> f64;
    fn set_light_source_direction(&mut self, direction: f64);
    fn get_light_source_altitude(&self) -> f64;
    fn set_light_source_altitude(&mut self, altitude: f64);
}

pub struct UpstreamKeyerDVESettings {
    pub border_enabled: bool,
    pub shadow_enabled: bool,
    pub border_bevel: enums::BorderBevel,
    pub rate: f64,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

impl UpstreamKeyerMaskSettings for UpstreamKeyerDVESettings {
    fn get_mask_enabled(&self) -> bool {
        self.mask_enabled
    }

    fn set_mask_enabled(&mut self, enabled: bool) {
        self.mask_enabled = enabled
    }

    fn get_mask_top(&self) -> f64 {
        self.mask_top
    }

    fn set_mask_top(&mut self, mask: f64) {
        self.mask_top = mask
    }

    fn get_mask_bottom(&self) -> f64 {
        self.mask_bottom
    }

    fn set_mask_bottom(&mut self, mask: f64) {
        self.mask_bottom = mask
    }

    fn get_mask_left(&self) -> f64 {
        self.mask_left
    }

    fn set_mask_right(&mut self, mask: f64) {
        self.mask_left = mask
    }
}

impl UpstreamKeyerDVEBase for UpstreamKeyerDVESettings {
    fn get_size_x(&self) -> f64 {
        self.size_x
    }

    fn set_size_x(&mut self, size_x: f64) {
        self.size_x = size_x
    }

    fn get_size_y(&self) -> f64 {
        self.size_y
    }

    fn set_size_y(&mut self, size_y: f64) {
        self.size_y = size_y
    }

    fn get_position_x(&self) -> f64 {
        self.position_x
    }

    fn set_position_x(&mut self, position_x: f64) {
        self.position_x = position_x
    }

    fn get_position_y(&self) -> f64 {
        self.position_y
    }

    fn set_position_y(&mut self, position_y: f64) {
        self.position_y = position_y
    }

    fn get_rotation(&self) -> f64 {
        self.rotation
    }

    fn set_rotation(&mut self, rotation: f64) {
        self.rotation = rotation
    }

    fn get_border_outer_width(&self) -> f64 {
        self.border_outer_width
    }

    fn set_border_outer_width(&mut self, width: f64) {
        self.border_outer_width = width
    }

    fn get_border_inner_width(&self) -> f64 {
        self.border_inner_width
    }

    fn set_border_inner_width(&mut self, width: f64) {
        self.border_inner_width = width
    }

    fn get_border_outer_softness(&self) -> f64 {
        self.border_outer_softness
    }

    fn set_border_outer_softness(&mut self, softness: f64) {
        self.border_outer_softness = softness
    }

    fn get_border_inner_softness(&self) -> f64 {
        self.border_inner_softness
    }

    fn set_border_inner_softness(&mut self, softness: f64) {
        self.border_inner_softness = softness
    }

    fn get_border_bevel_softness(&self) -> f64 {
        self.border_bevel_softness
    }

    fn set_border_bevel_softness(&mut self, softness: f64) {
        self.border_bevel_softness = softness
    }

    fn get_border_bevel_position(&self) -> f64 {
        self.border_bevel_position
    }

    fn set_border_bevel_position(&mut self, position: f64) {
        self.border_bevel_position = position
    }

    fn get_border_opacity(&self) -> f64 {
        self.border_opacity
    }

    fn set_border_opacity(&mut self, opacity: f64) {
        self.border_opacity = opacity
    }

    fn get_border_hue(&self) -> f64 {
        self.border_hue
    }

    fn set_border_hue(&mut self, hue: f64) {
        self.border_hue = hue
    }

    fn get_border_saturation(&self) -> f64 {
        self.border_saturation
    }

    fn set_border_saturation(&mut self, saturation: f64) {
        self.border_saturation = saturation
    }

    fn get_border_luma(&self) -> f64 {
        self.border_luma
    }

    fn set_border_luma(&mut self, luma: f64) {
        self.border_luma = luma
    }

    fn get_light_source_direction(&self) -> f64 {
        self.light_source_direction
    }

    fn set_light_source_direction(&mut self, direction: f64) {
        self.light_source_direction = direction
    }

    fn get_light_source_altitude(&self) -> f64 {
        self.light_source_altitude
    }

    fn set_light_source_altitude(&mut self, altitude: f64) {
        self.light_source_altitude = altitude
    }
}

pub struct UpstreamKeyerFlyKeyFrame {
    key_frame_id: f64,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

impl UpstreamKeyerMaskSettings for UpstreamKeyerFlyKeyFrame {
    fn get_mask_enabled(&self) -> bool {
        self.mask_enabled
    }

    fn set_mask_enabled(&mut self, enabled: bool) {
        self.mask_enabled = enabled
    }

    fn get_mask_top(&self) -> f64 {
        self.mask_top
    }

    fn set_mask_top(&mut self, mask: f64) {
        self.mask_top = mask
    }

    fn get_mask_bottom(&self) -> f64 {
        self.mask_bottom
    }

    fn set_mask_bottom(&mut self, mask: f64) {
        self.mask_bottom = mask
    }

    fn get_mask_left(&self) -> f64 {
        self.mask_left
    }

    fn set_mask_right(&mut self, mask: f64) {
        self.mask_left = mask
    }
}

impl UpstreamKeyerDVEBase for UpstreamKeyerFlyKeyFrame {
    fn get_size_x(&self) -> f64 {
        self.size_x
    }

    fn set_size_x(&mut self, size_x: f64) {
        self.size_x = size_x
    }

    fn get_size_y(&self) -> f64 {
        self.size_y
    }

    fn set_size_y(&mut self, size_y: f64) {
        self.size_y = size_y
    }

    fn get_position_x(&self) -> f64 {
        self.position_x
    }

    fn set_position_x(&mut self, position_x: f64) {
        self.position_x = position_x
    }

    fn get_position_y(&self) -> f64 {
        self.position_y
    }

    fn set_position_y(&mut self, position_y: f64) {
        self.position_y = position_y
    }

    fn get_rotation(&self) -> f64 {
        self.rotation
    }

    fn set_rotation(&mut self, rotation: f64) {
        self.rotation = rotation
    }

    fn get_border_outer_width(&self) -> f64 {
        self.border_outer_width
    }

    fn set_border_outer_width(&mut self, width: f64) {
        self.border_outer_width = width
    }

    fn get_border_inner_width(&self) -> f64 {
        self.border_inner_width
    }

    fn set_border_inner_width(&mut self, width: f64) {
        self.border_inner_width = width
    }

    fn get_border_outer_softness(&self) -> f64 {
        self.border_outer_softness
    }

    fn set_border_outer_softness(&mut self, softness: f64) {
        self.border_outer_softness = softness
    }

    fn get_border_inner_softness(&self) -> f64 {
        self.border_inner_softness
    }

    fn set_border_inner_softness(&mut self, softness: f64) {
        self.border_inner_softness = softness
    }

    fn get_border_bevel_softness(&self) -> f64 {
        self.border_bevel_softness
    }

    fn set_border_bevel_softness(&mut self, softness: f64) {
        self.border_bevel_softness = softness
    }

    fn get_border_bevel_position(&self) -> f64 {
        self.border_bevel_position
    }

    fn set_border_bevel_position(&mut self, position: f64) {
        self.border_bevel_position = position
    }

    fn get_border_opacity(&self) -> f64 {
        self.border_opacity
    }

    fn set_border_opacity(&mut self, opacity: f64) {
        self.border_opacity = opacity
    }

    fn get_border_hue(&self) -> f64 {
        self.border_hue
    }

    fn set_border_hue(&mut self, hue: f64) {
        self.border_hue = hue
    }

    fn get_border_saturation(&self) -> f64 {
        self.border_saturation
    }

    fn set_border_saturation(&mut self, saturation: f64) {
        self.border_saturation = saturation
    }

    fn get_border_luma(&self) -> f64 {
        self.border_luma
    }

    fn set_border_luma(&mut self, luma: f64) {
        self.border_luma = luma
    }

    fn get_light_source_direction(&self) -> f64 {
        self.light_source_direction
    }

    fn set_light_source_direction(&mut self, direction: f64) {
        self.light_source_direction = direction
    }

    fn get_light_source_altitude(&self) -> f64 {
        self.light_source_altitude
    }

    fn set_light_source_altitude(&mut self, altitude: f64) {
        self.light_source_altitude = altitude
    }
}

pub struct UpstreamKeyerChromaSettings {
    pub hue: f64,
    pub gain: f64,
    pub y_suppress: f64,
    pub lift: f64,
    pub narrow: bool,
}

pub struct UpstreamKeyerAdvancedChromaSettings {
    pub properties: Option<UpstreamKeyerAdvancedChromaProperties>,
    pub sample: Option<UpstreamKeyerAdvancedChromaSample>,
}

pub struct UpstreamKeyerAdvancedChromaProperties {
    pub foreground_level: f64,
    pub background_level: f64,
    pub key_edge: f64,

    pub spill_suppression: f64,
    pub flare_suppression: f64,

    pub brightness: f64,
    pub contrast: f64,
    pub saturation: f64,
    pub red: f64,
    pub green: f64,
    pub blue: f64,
}

pub struct UpstreamKeyerAdvancedChromaSample {
    pub enable_cursor: bool,
    pub preview: bool,
    pub cursor_x: f64,
    pub cursor_y: f64,
    pub cursor_size: f64,
    pub sampled_y: f64,
    pub sampled_cb: f64,
    pub sampled_cr: f64,
}

pub struct UpstreamKeyerLumaSettings {
    pub pre_multiplied: bool,
    pub clip: f64,
    pub gain: f64,
    pub invert: bool,
}

pub struct UpstreamKeyerPatternSettings {
    pub style: enums::Pattern,
    pub size: f64,
    pub symmetry: f64,
    pub softness: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub invert: bool,
}

pub struct UpstreamKeyerFlySettings {
    is_a_set: bool,
    is_b_set: bool,
    is_at_key_frame: enums::IsAtKeyFrame,
    run_to_infinite_index: f64,
}

pub struct UpstreamKeyer {
    pub mix_effect_key_type: enums::MixEffectKeyType,
    pub fly_enabled: bool,

    upstream_keyer_id: f64,
    can_fly_key: bool,
    pub fill_source: f64,
    pub cut_source: f64,

    pub dve_settings: Option<UpstreamKeyerDVESettings>,
    pub chroma_settings: Option<UpstreamKeyerChromaSettings>,
    pub advanced_chroma_settings: Option<UpstreamKeyerAdvancedChromaSettings>,
    pub luma_settings: Option<UpstreamKeyerLumaSettings>,
    pub pattern_settings: Option<UpstreamKeyerPatternSettings>,
    pub fly_keyframes: [Option<UpstreamKeyerFlyKeyFrame>; 2],
    pub fly_properties: Option<UpstreamKeyerFlySettings>,
    pub mask_settings: Box<dyn UpstreamKeyerMaskSettings>,
    pub on_air: bool,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

impl UpstreamKeyerTypeSettings for UpstreamKeyer {
    fn get_mix_effect_key_type(&self) -> enums::MixEffectKeyType {
        self.mix_effect_key_type
    }

    fn set_mix_effect_key_type(&mut self, key_type: enums::MixEffectKeyType) {
        self.mix_effect_key_type = key_type
    }

    fn get_fly_enabled(&self) -> bool {
        self.fly_enabled
    }

    fn set_fly_enabled(&mut self, enabled: bool) {
        self.fly_enabled = enabled
    }
}

impl UpstreamKeyerBase for UpstreamKeyer {
    fn get_upstream_keyer_id(&self) -> f64 {
        self.upstream_keyer_id
    }

    fn get_can_fly_key(&self) -> bool {
        self.can_fly_key
    }

    fn get_fill_source(&self) -> f64 {
        self.fill_source
    }

    fn set_fill_source(&mut self, source: f64) {
        self.fill_source = source
    }

    fn get_cut_source(&self) -> f64 {
        self.cut_source
    }

    fn set_cut_source(&mut self, source: f64) {
        self.cut_source = source
    }
}
