use std::time::Duration;

use atem_connection_rs::{
    atem_lib::atem_socket::AtemSocket,
    commands::{
        command_base::{BasicWritableCommand, SerializableCommand},
        mix_effects::ProgramInput,
    },
};

use color_eyre::Report;
use tokio::time::sleep;

#[tokio::main]
async fn main() {
    setup_logging().unwrap();

    let switch_to_source_1 = ProgramInput::new(0, 1);
    let switch_to_source_2 = ProgramInput::new(0, 2);

    let mut atem = AtemSocket::new();
    atem.connect("127.0.0.1".to_string(), 9910).await.ok();

    let mut tracking_id = 0;
    loop {
        sleep(Duration::from_millis(5000)).await;
        atem.send_command(
            &switch_to_source_1.payload(atem_connection_rs::enums::ProtocolVersion::Unknown),
            switch_to_source_1.get_raw_name(),
            tracking_id,
        )
        .await;
        tracking_id += 1;
        sleep(Duration::from_millis(5000)).await;
        atem.send_command(
            &switch_to_source_2.payload(atem_connection_rs::enums::ProtocolVersion::Unknown),
            switch_to_source_2.get_raw_name(),
            tracking_id,
        )
        .await;
        tracking_id += 1;
    }
}

fn setup_logging() -> Result<(), Report> {
    if std::env::var("RUST_LIB_BACKTRACE").is_err() {
        std::env::set_var("RUST_LIB_BACKTRACE", "1");
    }
    color_eyre::install()?;

    if std::env::var("RUST_LOG").is_err() {
        std::env::set_var("RUST_LOG", "debug");
    }
    env_logger::init();

    Ok(())
}
